package app

import (
	"fmt"
	"os"
	"os/exec"
	"path"
	"strconv"

	"git.sr.ht/~rockorager/vaxis"
	"git.sr.ht/~rockorager/vaxis/widgets/term"
	"git.tilde.town/nbsp/neofeels/config"
	"git.tilde.town/nbsp/neofeels/ttbp"
	"git.tilde.town/nbsp/neofeels/ui"
)

type UserPage struct {
	title string
	list  ui.List
	help  string
	posts []ttbp.Post
	self  bool
	index int // which index on the previous page to return to?
}

func NewUserPage(user string, self bool, index int) *UserPage {
	posts := ttbp.GetPostsForUser(user)
	list := []string{}
	for _, post := range posts {
		list = append(list, fmt.Sprintf(
			"%s (%d words)",
			post.Date.Format("2006-01-02"),
			post.Words,
		))
	}

	return &UserPage{
		title,
		ui.NewList(list, 0),
		"↑↓/kj move  ↵ enter  q return",
		posts,
		self,
		index,
	}
}

func (user *UserPage) Event(state *ui.State, event vaxis.Event) (processed bool) {
	if key, ok := event.(vaxis.Key); ok && key.EventType == vaxis.EventPress {
		switch key.String() {
		case "Ctrl+c", "Ctrl+d":
			close(ui.Quit)
		case "Down", "j", "Ctrl+n":
			user.list.Down()
		case "Up", "k", "Ctrl+p":
			user.list.Up()
		case "End", "Shift+g":
			user.list.End()
		case "Home", "g":
			user.list.Home()
		case "0", "1", "2", "3", "4", "5", "6", "7", "8", "9":
			i, _ := strconv.Atoi(key.String())
			user.list.SetIndex(i)
		case "q", "h", "Left":
			if user.self {
				ui.ViewChange <- NewManagement(0)
			} else {
				ui.ViewChange <- NewNeighbors(user.index)
			}
		case "Enter", "l", "Right":
			showPost(state, user.posts[user.list.Index()])
		}
		processed = true
	}
	user.Draw(state)
	return
}

func (user *UserPage) Draw(state *ui.State) {
	win := state.Window()
	win.New(win.Width/2-10, win.Height/2-8, 20, 5).Print(vaxis.Segment{Text: user.title})
	user.list.Draw(vaxis.Window{
		Vx:     win.Vx,
		Parent: nil,
		Column: win.Width/2 - 14,
		Row:    win.Height/2 - 2,
		Width:  28,
		Height: 10,
	})
	win.New(win.Width/2-15, win.Height/2+9, 30, 1).Print(vaxis.Segment{Text: user.help})
}

func showPost(state *ui.State, post ttbp.Post) {
	state.HideCursor()
	vt := term.New()
	vt.TERM = os.Getenv("TERM")
	vt.Attach(state.PostEvent())
	vt.Focus()

	cfg, err := config.Read()
	if err != nil {
		panic(err)
	}
	pager := os.ExpandEnv(cfg.Pager)
	if cfg.Pager == "" {

		pager = os.ExpandEnv(os.Getenv("PAGER"))
	}
	if pager == "" {
		pager = "less"
	}

	prepro := "cat"
	if post.HTML {
		prepro = "lynx -dump -force_html"
	}

	cmd := fmt.Sprintf("%s %s | %s", prepro, path.Join("/home", post.Author, ".ttbp/entries", post.Date.Format("20060102")+".txt"), pager)
	err = vt.Start(exec.Command("sh", "-c", cmd))
	if err != nil {
		panic(err)
	}
	defer vt.Close()

	for ev := range state.Events() {
		switch ev.(type) {
		case term.EventClosed:
			state.HideCursor()
			state.Window().Clear()
			return
		case vaxis.Redraw:
			vt.Draw(state.Window())
			state.Render()
			continue
		}

		vt.Update(ev)
	}
}
